import time

from .helper import WaitForTopic

class Action(object):
    SUCCESS = 0
    FAILED = 1

    def setup(self, proxy):
        """
        proxy: a thread safe proxy object with publish, subscribe method
        """
        self.proxy = proxy
        return self

    def ok(self):
        return Action.SUCCESS

    def failed(self):
        return Action.FAILED

    @property
    def name(self):
        return self.__class__.__name__


class Publish(Action):

    def __init__(self, topic: str):
        Action.__init__(self)
        self.topic = topic
        self.payload = ''
        self.qos = 0

        self.check_response = None

    def Payload(self, payload):
        self.payload = payload
        return self

    def Qos(self, qos):
        self.qos = qos
        return self

    def CheckResponse(self, topic: str, payload: str=None, timeout: int=None):
        self.check_response= {'topic': topic, 'payload':payload, 'timeout': timeout}
        return self

    def run(self):
        checker = None
        if self.check_response:
            checker = WaitForTopic(self.proxy.configuration,
                    self.check_response['topic'],
                    self.check_response['payload'])

        self.proxy.threaded_publish(self.topic,
                payload=self.payload, qos=self.qos, retain=False)

        if self.check_response:
            checker.join(timeout=self.check_response['timeout'])
            if checker.is_alive(): #timeout occur
                checker.stop()
                return self.failed()

        return self.ok()

class TasmotaShutter(Action):

    TOPIC_FORMAT = "cmnd/tasmota/shutter/{shutter_name}/{action}"
    TOPIC_CHECK_FORMAT = "stat/tasmota/shutter/{shutter_name}/RESULT"
    ACTION_OPEN = 'ShutterOpen'
    ACTION_CLOSE = 'ShutterClose'

    def __init__(self, shutter_name: str):
        Action.__init__(self)
        self.shutter_name = shutter_name
        self.action = None

    def Close(self):
        self.action = self.ACTION_CLOSE
        return self

    def Open(self):
        self.action = self.ACTION_OPEN
        return self

    def run(self):
        if self.action is None:
            raise RuntimeError('No action specified')

        topic = self.TOPIC_FORMAT.format(
            shutter_name = self.shutter_name,
            action = self.action
            )

        checker = WaitForTopic(self.proxy.configuration,
                self.TOPIC_CHECK_FORMAT.format(shutter_name = self.shutter_name),
                payload= '{"Shutter1":{"Position":100,"Direction":0,"Target":100}}' \
                        if self.action == self.ACTION_OPEN \
                        else '{"Shutter1":{"Position":0,"Direction":0,"Target":0}}')
        time.sleep(1)
        self.proxy.threaded_publish(topic, qos=1, retain=False)

        checker.join(timeout=35)
        if checker.is_alive():
            return self.failed()

        return self.ok()
