import threading
import logging
from typing import Union, Callable

import paho.mqtt.client as mqtt

from citadel.mqtt import Configuration

class WaitForTopic(threading.Thread, mqtt.Client):

    def __init__(self,
            config: Configuration,
            topic: str,
            payload: Union[str, Callable[[str],bool]],
            logger: logging.Logger=None):
        """ payload: str or function stop if payload is received or function return True, the function must have this signature func(payload: str) -> bool """

        threading.Thread.__init__(self)
        mqtt.Client.__init__(self)

        self.configuration = config

        self.topic = topic

        if callable(payload):
            self.check_payload = payload
        else:
            self.check_payload = lambda x: x == payload

        self.daemon = True

        if not logger:
            logger = logging.getLogger('watcher')
        self.logger = logger

        self.__run = True

        self.username_pw_set(self.configuration.user,
                self.configuration.password)
        self.connect(self.configuration.host,
                self.configuration.port)

        self.subscribe(self.topic, 1)

        self.start()

    def on_message(self, mqttc, obj, msg):
        payload = msg.payload.decode()
        topic = msg.topic
        self.logger.debug('Topic: %s, Payload: %s', topic, payload)

        if topic != self.topic:
            self.logger.error('Invaliad topic: %s', topic)
            return

        if self.check_payload(payload):
            self.__run = False

    def stop(self):
        self.__run = False

    def run(self):
        while self.__run:
            self.loop()
        self.disconnect()
